@@ -3,6 +3,8 @@ import codecs
 import logging
 
 from django import forms
+from django.contrib.auth.forms import UserCreationForm
+from django.forms.models import model_to_dict
 from django.utils.translation import gettext_lazy as _
 from django.db import IntegrityError
 from django.core.exceptions import ValidationError
@@ -13,6 +15,7 @@ from .constants import VALUE_UNITS, PACKAGE_TYPES, POWER_UNITS, INTERFACE_TYPES,
     WEIGHT_UNITS, FREQUENCY_UNITS, VOLTAGE_UNITS, CURRENT_UNITS, MEMORY_UNITS, SUBSCRIPTION_TYPES, ROLE_TYPES, CONFIGURATION_TYPES, NUMBER_SCHEME_SEMI_INTELLIGENT, \
     NUMBER_SCHEME_INTELLIGENT, ROLE_TYPE_VIEWER, NUMBER_CLASS_CODE_LEN_DEFAULT, NUMBER_CLASS_CODE_LEN_MIN, NUMBER_CLASS_CODE_LEN_MAX, \
     NUMBER_ITEM_LEN_DEFAULT, NUMBER_ITEM_LEN_MIN, NUMBER_ITEM_LEN_MAX, NUMBER_VARIATION_LEN_DEFAULT, NUMBER_VARIATION_LEN_MIN, NUMBER_VARIATION_LEN_MAX
+from .form_fields import AutocompleteTextInput
 from .models import Part, PartClass, Manufacturer, ManufacturerPart, Subpart, Seller, SellerPart, User, UserMeta, \
     Organization, PartRevision, AssemblySubparts, Assembly
 from .validators import decimal, numeric, alphanumeric
@@ -34,13 +37,25 @@ class UserModelChoiceField(forms.ModelChoiceField):
         return l
 
 
-class UserCreateForm(forms.ModelForm):
-    class Meta:
-        model = User
-        fields = ('username', 'first_name', 'last_name', 'email', 'password',)
-        widgets = {
-            'password': forms.PasswordInput(),
-        }
+class UserCreateForm(UserCreationForm):
+    first_name = forms.CharField(required=True)
+    last_name = forms.CharField(required=True)
+    email = forms.EmailField(required=True)
+
+    def clean_email(self):
+        email = self.cleaned_data['email']
+        exists = User.objects.filter(email__iexact=email).count() > 0
+        if exists:
+            raise ValidationError('An account with this email address already exists.')
+        return email
+
+    def save(self, commit=True):
+        user = super(UserCreateForm, self).save(commit=commit)
+        user.email = self.cleaned_data['email']
+        user.first_name = self.cleaned_data['first_name']
+        user.last_name = self.cleaned_data['last_name']
+        user.save()
+        return user
 
 
 class UserForm(forms.ModelForm):
@@ -204,12 +219,15 @@ class ManufacturerPartForm(forms.ModelForm):
         model = ManufacturerPart
         exclude = ['part', ]
 
+    field_order = ['manufacturer_part_number', 'manufacturer']
+
     def __init__(self, *args, **kwargs):
         self.organization = kwargs.pop('organization', None)
         super(ManufacturerPartForm, self).__init__(*args, **kwargs)
         self.fields['manufacturer'].required = False
         self.fields['manufacturer_part_number'].required = False
         self.fields['manufacturer'].queryset = Manufacturer.objects.filter(organization=self.organization).order_by('name')
+        self.fields['mouser_disable'].initial = True
 
 
 class SellerPartForm(forms.ModelForm):
@@ -302,8 +320,25 @@ class PartClassSelectionForm(forms.Form):
     def __init__(self, *args, **kwargs):
         self.organization = kwargs.pop('organization', None)
         super(PartClassSelectionForm, self).__init__(*args, **kwargs)
-        self.fields['part_class'] = forms.ModelChoiceField(queryset=PartClass.objects.filter(organization=self.organization).order_by('code'),
-                                                           empty_label="- Select Part Class -", label='List parts by class', required=False)
+        self.fields['part_class'] = forms.CharField(required=False,
+                                                    widget=AutocompleteTextInput(attrs={'placeholder': 'Select a part class.'},
+                                                                                 autocomplete_submit=True,
+                                                                                 queryset=PartClass.objects.filter(organization=self.organization)))
+
+    def clean_part_class(self):
+        part_class = self.cleaned_data['part_class']
+        if part_class == '':
+            return None
+
+        try:
+            return PartClass.objects.get(organization=self.organization, code=part_class.split(':')[0])
+        except PartClass.DoesNotExist:
+            pc = PartClass.objects.filter(name__icontains=part_class).order_by('name').first()
+            if pc is not None:
+                return pc
+            else:
+                self.add_error('part_class', 'Select a valid part class.')
+        return None
 
 
 class PartClassCSVForm(forms.Form):
@@ -324,7 +359,7 @@ class PartClassCSVForm(forms.Form):
             dialect = csv.Sniffer().sniff(csvline_decoded)
             file.open()
             reader = csv.reader(codecs.iterdecode(file, 'utf-8'), dialect)
-            headers = [h.lower() for h in next(reader)]
+            headers = [h.lower().replace('\ufeff', '') for h in next(reader)]
 
             csv_headers = PartClassesCSVHeaders()
 
@@ -420,7 +455,12 @@ class PartCSVForm(forms.Form):
             reader = csv.reader(codecs.iterdecode(file, 'utf-8'), dialect)
             headers = [h.lower() for h in next(reader)]
 
-            csv_headers = PartsListCSVHeaders()
+            # Handle utf-8-sig encoding
+            if "\ufeff" in headers[0]:
+                reader = csv.reader(codecs.iterdecode(file, 'utf-8-sig'), dialect)
+                headers = [h.lower() for h in next(reader)]
+
+            csv_headers = self.organization.part_list_csv_headers()
 
             try:
                 # Issue warning if unrecognized column header names appear in file.
@@ -602,17 +642,39 @@ class PartCSVForm(forms.Form):
                 if skip:
                     continue
 
-                part = Part.objects.create(number_class=part_class, number_item=number_item, number_variation=number_variation, organization=self.organization)
-                part_revision.part = part
-                if mfg_name and mpn:
-                    mfg, created = Manufacturer.objects.get_or_create(name__iexact=mfg_name, organization=self.organization, defaults={'name': mfg_name})
-                    manufacturer_part, created = ManufacturerPart.objects.get_or_create(part=part, manufacturer_part_number=mpn, manufacturer=mfg)
-                    if part.primary_manufacturer_part is None and manufacturer_part is not None:
-                        part.primary_manufacturer_part = manufacturer_part
+                PartForm = PartFormSemiIntelligent if self.organization.number_scheme == NUMBER_SCHEME_SEMI_INTELLIGENT else PartFormIntelligent
+                part = Part(number_class=part_class, number_item=number_item, number_variation=number_variation, organization=self.organization)
+                part_dict = model_to_dict(part)
+                part_dict.update({'number_class': str(part.number_class)})
+                pf = PartForm(data=part_dict, organization=self.organization)
+                prf = PartRevisionForm(data=model_to_dict(part_revision))
 
-                part.save()
-                part_revision.save()
-                self.successes.append("Part {0} on row {1} created.".format(part.full_part_number(), row_count))
+                if pf.is_valid() and prf.is_valid():
+                    part = pf.save(commit=False)
+                    part.organization = self.organization
+                    part.save()
+                    part_revision = prf.save(commit=False)
+                    part_revision.part = part
+                    part_revision.save()
+
+                    if mfg_name and mpn:
+                        mfg, created = Manufacturer.objects.get_or_create(name__iexact=mfg_name, organization=self.organization, defaults={'name': mfg_name})
+                        manufacturer_part, created = ManufacturerPart.objects.get_or_create(part=part, manufacturer_part_number=mpn, manufacturer=mfg)
+                        if part.primary_manufacturer_part is None and manufacturer_part is not None:
+                            part.primary_manufacturer_part = manufacturer_part
+
+                    self.successes.append("Part {0} on row {1} created.".format(part.full_part_number(), row_count))
+                else:
+                    for k, error in prf.errors.items():
+                        for idx, msg in enumerate(error):
+                            error[idx] = f"Error on Row {row_count}, {k}: " + msg
+                        self.errors.update({k: error})
+                    for k, error in pf.errors.items():
+                        for idx, msg in enumerate(error):
+                            error[idx] = f"Error on Row {row_count}, {k}: " + msg
+                        self.errors.update({k: error})
+
+                # part = Part.objects.create(number_class=part_class, number_item=number_item, number_variation=number_variation, organization=self.organization)
 
         except UnicodeDecodeError as e:
             self.add_error(None, forms.ValidationError("CSV File Encoding error, try encoding your file as utf-8, and upload again. \
@@ -650,7 +712,6 @@ class PartFormSemiIntelligent(forms.ModelForm):
         model = Part
         exclude = ['organization', 'google_drive_parent', ]
         help_texts = {
-            'number_class': _('Select a number class.'),
             'number_item': _('Auto generated if blank.'),
             'number_variation': 'Auto generated if blank.',
         }
@@ -659,8 +720,8 @@ class PartFormSemiIntelligent(forms.ModelForm):
         self.organization = kwargs.pop('organization', None)
         super(PartFormSemiIntelligent, self).__init__(*args, **kwargs)
         self.fields['number_item'].validators.append(alphanumeric)
-        self.fields['number_class'] = forms.ModelChoiceField(queryset=PartClass.objects.filter(organization=self.organization),
-                                                             empty_label="- Select Part Number Class -", label='Part Number Class*', required=True)
+        self.fields['number_class'] = forms.CharField(label='Part Number Class*', required=True, help_text='Select a number class.',
+                                                      widget=AutocompleteTextInput(queryset=PartClass.objects.filter(organization=self.organization)))
         if self.instance and self.instance.id:
             self.fields['primary_manufacturer_part'].queryset = ManufacturerPart.objects.filter(part__id=self.instance.id).order_by('manufacturer_part_number')
         else:
@@ -669,6 +730,14 @@ class PartFormSemiIntelligent(forms.ModelForm):
             value.widget.attrs['placeholder'] = value.help_text
             value.help_text = ''
 
+    def clean_number_class(self):
+        number_class = self.cleaned_data['number_class']
+        try:
+            return PartClass.objects.get(organization=self.organization, code=number_class.split(':')[0])
+        except PartClass.DoesNotExist:
+            self.add_error('number_class', f'Select an existing part class, or create `{number_class}` in Settings.')
+        return None
+
     def clean(self):
         cleaned_data = super(PartFormSemiIntelligent, self).clean()
         number_item = cleaned_data.get('number_item')
@@ -704,7 +773,7 @@ class PartFormSemiIntelligent(forms.ModelForm):
                 organization=self.organization
             )
             validation_error = forms.ValidationError(
-                ("Part number {0}-{1}-{2} already in use.".format(number_class, number_item, number_variation)),
+                ("Part number {0}-{1}-{2} already in use.".format(number_class.code, number_item, number_variation)),
                 code='invalid')
             self.add_error(None, validation_error)
         except Part.DoesNotExist:
@@ -847,6 +916,9 @@ class AddSubpartForm(forms.Form):
         self.unusable_part_rev_ids = [pr.id for pr in self.part_revision.where_used_full()]
         self.unusable_part_rev_ids.append(self.part_revision.id)
         super(AddSubpartForm, self).__init__(*args, **kwargs)
+        self.fields['subpart_part_number'] = forms.CharField(required=True, label="Subpart part number",
+                                                    widget=AutocompleteTextInput(attrs={'placeholder': 'Select a part.'},
+                                                                                 queryset=Part.objects.filter(organization=self.organization).exclude(id=self.part_id)))
 
     def clean_count(self):
         count = self.cleaned_data['count']
@@ -873,10 +945,13 @@ class AddSubpartForm(forms.Form):
             if self.organization.number_scheme == NUMBER_SCHEME_INTELLIGENT:
                 part = Part.objects.get(number_item=subpart_part_number, organization=self.organization)
             else:
-                (number_class, number_item, number_variation) = Part.parse_partial_part_number(subpart_part_number, self.organization)
+                (number_class, number_item, number_variation) = Part.parse_partial_part_number(subpart_part_number, self.organization, validate=False)
                 part_class = PartClass.objects.get(code=number_class, organization=self.organization)
                 part = Part.objects.get(number_class=part_class, number_item=number_item, number_variation=number_variation, organization=self.organization)
             self.subpart_part = part.latest()
+            if self.subpart_part is None:
+                self.add_error('subpart_part_number', f"No part revision exists for part {part.full_part_number()}. Create a revision before adding to an assembly.")
+                return subpart_part_number
             if self.subpart_part.id in self.unusable_part_rev_ids:
                 validation_error = forms.ValidationError("Infinite recursion! Can't add a part to its self.", code='invalid')
                 self.add_error('subpart_part_number', validation_error)
