@@ -30,4 +30,7 @@ DATABASES = {
 }
 
 EMAIL_BACKEND = "sendgrid_backend.SendgridBackend"
-SENDGRID_API_KEY = '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'
\ No newline at end of file
+SENDGRID_API_KEY = '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'
+
+# For django-money exchange rates
+FIXER_ACCESS_KEY = '1a3d87f692c8310e791bfde536f1643e'
\ No newline at end of file
