@@ -7,13 +7,15 @@ from django.db import models
 from django.contrib.auth.models import User, Group
 from django.utils import timezone
 from django.core.validators import MaxValueValidator, MinValueValidator
+from .csv_headers import PartsListCSVHeadersSemiIntelligent, PartsListCSVHeaders
 from .part_bom import PartIndentedBomItem, PartBomItem, PartBom
 from .utils import increment_str, prep_for_sorting_nicely, listify_string, stringify_list, strip_trailing_zeros
 from .validators import alphanumeric, numeric, validate_pct
 from .constants import VALUE_UNITS, PACKAGE_TYPES, POWER_UNITS, INTERFACE_TYPES, TEMPERATURE_UNITS, DISTANCE_UNITS, WAVELENGTH_UNITS, \
     WEIGHT_UNITS, FREQUENCY_UNITS, VOLTAGE_UNITS, CURRENT_UNITS, MEMORY_UNITS, SUBSCRIPTION_TYPES, ROLE_TYPES, CONFIGURATION_TYPES, \
     NUMBER_SCHEMES, NUMBER_SCHEME_SEMI_INTELLIGENT, NUMBER_CLASS_CODE_LEN_DEFAULT, NUMBER_CLASS_CODE_LEN_MIN, NUMBER_CLASS_CODE_LEN_MAX, \
-    NUMBER_ITEM_LEN_DEFAULT, NUMBER_ITEM_LEN_MIN, NUMBER_ITEM_LEN_MAX, NUMBER_VARIATION_LEN_DEFAULT, NUMBER_VARIATION_LEN_MIN, NUMBER_VARIATION_LEN_MAX
+    NUMBER_ITEM_LEN_DEFAULT, NUMBER_ITEM_LEN_MIN, NUMBER_ITEM_LEN_MAX, NUMBER_VARIATION_LEN_DEFAULT, NUMBER_VARIATION_LEN_MIN, NUMBER_VARIATION_LEN_MAX, \
+    NUMBER_SCHEME_INTELLIGENT
 from .base_classes import AsDictModel
 
 from djmoney.models.fields import MoneyField, CurrencyField, CURRENCY_CHOICES
@@ -52,6 +54,12 @@ class Organization(models.Model):
     def seller_parts(self):
         return SellerPart.objects.filter(seller__organization=self)
 
+    def part_list_csv_headers(self):
+        if self.number_scheme == NUMBER_SCHEME_INTELLIGENT:
+            return PartsListCSVHeaders()
+        else:
+            return PartsListCSVHeadersSemiIntelligent()
+
     def save(self, *args, **kwargs):
         super(Organization, self).save()
         SellerPart.objects.filter(seller__organization=self).update(unit_cost_currency=self.currency, nre_cost_currency=self.currency)
@@ -102,7 +110,7 @@ class PartClass(models.Model):
         index_together = [['organization', 'code', ], ]
 
     def __str__(self):
-        return u'%s' % (self.code + ': ' + self.name)
+        return f'{self.code}: {self.name}'
 
 
 class Manufacturer(models.Model, AsDictModel):
@@ -168,7 +176,7 @@ class Part(models.Model):
         elif number_variation is not None:
             for c in number_variation:
                 if not c.isalnum():
-                    raise AttributeError(f"{c} is not a proper character for a number variation")
+                    raise AttributeError(f"{c} is not a proper character for a number variation. Must be alphanumeric.")
         return number_variation
 
     @staticmethod
@@ -176,7 +184,10 @@ class Part(models.Model):
         if part_number is None:
             raise AttributeError("Cannot parse empty part number")
 
-        (number_class, number_item, number_variation) = Part.parse_partial_part_number(part_number, organization)
+        try:
+            (number_class, number_item, number_variation) = Part.parse_partial_part_number(part_number, organization)
+        except IndexError:
+            raise AttributeError("Invalid part number. Does not match organization preferences.")
 
         if number_class is None:
             raise AttributeError("Missing part number part class")
@@ -188,19 +199,19 @@ class Part(models.Model):
         return number_class, number_item, number_variation
 
     @staticmethod
-    def parse_partial_part_number(part_number, organization):
+    def parse_partial_part_number(part_number, organization, validate=True):
         elements = part_number.split('-')
-        number_class = None
-        number_item = None
-        number_variation = None
 
-        if len(elements) == 3:
-            number_class = Part.verify_format_number_class(elements[0], organization)
-            number_item = Part.verify_format_number_item(elements[1], organization)
-            number_variation = Part.verify_format_number_variation(elements[2], organization)
-        elif len(elements) == 2:
-            number_class = Part.verify_format_number_class(elements[0], organization)
-            number_item = Part.verify_format_number_item(elements[1], organization)
+        number_class = elements[0] if len(elements) >= 1 else None
+        number_item = elements[1] if len(elements) >= 2 else None
+        number_variation = elements[2] if len(elements) >= 3 else None
+
+        if validate:
+            if len(elements) >= 2:
+                number_class = Part.verify_format_number_class(elements[0], organization)
+                number_item = Part.verify_format_number_item(elements[1], organization)
+            if len(elements) >= 3:
+                number_variation = Part.verify_format_number_variation(elements[2], organization)
 
         return number_class, number_item, number_variation
 
